Package: lorentz
Type: Package
Title: The Lorentz Transform in Relativistic Physics
Version: 1.1-2
Authors@R: person(given=c("Robin", "K. S."), family="Hankin", role = c("aut","cre"), email="hankin.robin@gmail.com", comment = c(ORCID = "0000-0001-5982-0415"))
Suggests: knitr,testthat,rmarkdown,covr
Imports: quadform,tensor,magic,magrittr
Maintainer: Robin K. S. Hankin <hankin.robin@gmail.com>
Depends: R (>= 4.1.0)
Description: The Lorentz transform in special relativity; also the
    gyrogroup structure of three-velocities.  Performs active and
    passive transforms and has the ability to use units in which the
    speed of light is not unity.  Includes some experimental
    functionality for celerity and rapidity.  For general relativity,
    see the 'schwarzschild' package.
License: GPL-3
Encoding: UTF-8
VignetteBuilder: knitr
URL: https://github.com/RobinHankin/lorentz,
        https://robinhankin.github.io/lorentz/
BugReports: https://github.com/RobinHankin/lorentz/issues
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2025-01-22 11:42:42 UTC; rhankin
Author: Robin K. S. Hankin [aut, cre] (<https://orcid.org/0000-0001-5982-0415>)
Repository: CRAN
Date/Publication: 2025-01-23 16:20:01 UTC
Built: R 4.6.0; ; 2026-04-26 21:24:17 UTC; unix
